import time
import uuid
from datetime import datetime, timedelta


def dt_time(sub=8):
    sub = int(sub)
    nnn = time.time() - sub * 60 * 60
    dat = datetime.fromtimestamp(nnn)
    ttt = dat.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
    return ttt


def dt_now_time():
    nnn = time.time()
    dat = datetime.fromtimestamp(nnn)
    ttt = dat.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
    return ttt


def get_uuid():
    uid = str(uuid.uuid4())
    suid = "".join(uid.split("-"))
    return suid


def to_utc_time(_time):
    if '+' in _time:
        times = _time.split("+")
        _time = times[0]
        v = times[1]
        f_srt = '%Y-%m-%dT%H:%M:%S.%f' if '.' in _time else '%Y-%m-%dT%H:%M:%S'
        _time = datetime.strptime(_time, f_srt) - timedelta(hours=int(v.split(':')[0]))
        _time = _time.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + "Z"
    elif ' ' in _time:
        f_srt = '%Y-%m-%d %H:%M:%S.%f' if '.' in _time else '%Y-%m-%d %H:%M:%S'
        _time = datetime.strptime(_time, f_srt) - timedelta(hours=8)
        _time = _time.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + "Z"
    return _time


def convert_time(_time):
    if '+' in _time:
        times = _time.split("+")
        _time = times[0]
        v = times[1]
        f_srt = '%Y-%m-%dT%H:%M:%S.%f' if '.' in _time else '%Y-%m-%dT%H:%M:%S'
        _time = datetime.strptime(_time, f_srt) + timedelta(hours=8 - int(v.split(':')[0]))
        _time = _time.strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]
    elif 'Z' in _time:
        f_srt = '%Y-%m-%dT%H:%M:%S.%fZ' if '.' in _time else '%Y-%m-%dT%H:%M:%SZ'
        _time = datetime.strptime(_time, f_srt) + timedelta(hours=8)
        _time = _time.strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]
    return _time


if __name__ == '__main__':
    print(to_utc_time('2021-10-21 03:30:04.449'))



class BaseArgsParsing:
    PAGE = "page"
    SIZE = "size"
    PAGESIZE = "pagesize"
    SORT = "sort"
    ASC = "asc"
    DESC = "desc"
    TOKEN = 'token'
    IGNORE_CREATOR = 'ignore_creator'
    ALL_RESULT = 'all_result'
    FILTER = 'filter'

    LIKE = "like"
    NOT = "not"
    IN = "in"
    LT = "lt"
    GT = "gt"
    LTE = "lte"
    GTE = "gte"
    ISNULL = 'isnull'
    OR = 'or'

    RESERVED_FIELD = [PAGE, SIZE, PAGESIZE, SORT, TOKEN, IGNORE_CREATOR, ALL_RESULT, FILTER]
    SUFFIX_KEYWORD = [LIKE, NOT, IN, LT, GT, LTE, GTE, ISNULL, OR]

    def get_page_and_size(self, data, max_page=500, max_size=1000, max_total=10000):
        """
        获取页面和条数
        :param data:
        :return:
        """
        try:
            page = int(data.get(self.PAGE, 1))
            size = int(data.get(self.SIZE, 20))
        except Exception as e:
            raise ValueError(f"'{self.PAGE}' or '{self.SIZE}' must be int")

        if page * size > max_total:
            raise ValueError(
                f"page*size has to be less than max_total,page='{page}',size='{size}',max_total={max_total}"
            )
        if data.get(self.PAGESIZE):
            size = int(data.get(self.PAGESIZE))
        page = page if page < max_page else max_page
        size = size if size < max_size else max_size

        return page, size

    def get_sort(self, data, default_field=None, default_sort=None):
        item = data.get(self.SORT)
        field = default_field
        sort = default_sort
        assert (default_field and default_sort) or (
                (not default_field) and (not default_sort)
        ), f"default_field={default_field}, default_sort={default_sort}"
        if item:
            item = item.lower().strip().split(" ")
            if len(item) <= 1:
                raise ValueError(f"'{self.SORT}'  format like  'sort=field asc'")
            field = item[0]
            sort = item[-1].lower()
            if sort != self.ASC and sort != self.DESC:
                raise ValueError(f"'{self.SORT}' must be  '{self.ASC}'  or '{self.DESC}'")
        return field, sort



from copy import deepcopy

from django.db.models import Count
from django.db.models.query import QuerySet
from rest_framework.exceptions import ValidationError
from common.rest_extend.response import RESTResponse, Results, SUCCESS, get_error_status_code, REQUEST_ERROR_CODE
from common.rest_extend.decorator import related_vaild_legal_decorator
from rest_framework.views import APIView
from common.rest_extend.views.views import Select, BaseView


class InjectionData:
    def related_data(self, method):
        related_obj = None
        related_mapping = None
        return related_obj, related_mapping

    def m_to_m_data(self, method):
        m_to__obj = None
        m_to__mapping = None
        return m_to__obj, m_to__mapping


class Related(Select):
    def execute_related_extend(self, related_obj, related_mapping, data):
        if related_obj and related_mapping:
            for obj in related_obj:
                obj_name = obj.__name__.lower()
                obj_mapping = related_mapping.get(obj_name)
                if not obj_mapping:
                    raise Exception(f"{obj_name}  are not mapping!!!")
                queryset = self.get_relevance_queryset(obj, obj_name, data)
                self.extend_relevance_data(data, queryset, obj_mapping)

    def execute_m_to_m_extend(self, m_to_m_obj, m_to_m_mapping, data):
        pass

    def extend_relevance_data(self, data, queryset, obj_mapping):
        if queryset:
            value = getattr(queryset, obj_mapping[0], None)
            if value is not None:
                data[obj_mapping[1]] = value

    def get_relevance_orm_q(self, obj_name, data):
        conditions = {}
        if data:
            for item, value in data.items():
                if obj_name + "__" in item:
                    conditions[item.replace(obj_name + "__", "")] = value
            if conditions:
                orm_q = self.get_orm_q(conditions)
                return orm_q

    def get_relevance_queryset(self, obj, obj_name, data):
        orm_q = self.get_relevance_orm_q(obj_name, data)
        if orm_q:
            queryset = obj.objects.filter(orm_q).first()
            return queryset

    def extend_result_field(self, obj_serializer, related_obj, m_to_m_obj, related_mapping, m_to_m_mapping, queryset):
        """
        扩展字段
        1、获取关联表
        2、遍历结果集、进行字段扩展
        :param obj_relevance:
        :param mapping:
        :param queryset:
        :return:
        """

        result = []
        # 遍历结果集
        for item in queryset:
            item_result = item if isinstance(item, dict) else dict(obj_serializer(item).data)
            self.extend_m_to_m_field(item, item_result, m_to_m_obj, m_to_m_mapping)
            self.extend_related_field(item, item_result, related_obj, related_mapping)
            result.append(item_result)
        return result

    def extend_m_to_m_field(self, item, item_result, m_to_m_obj, m_to_m_mapping):
        if not m_to_m_obj:
            return
        for obj in m_to_m_obj:
            m_to_m_name = obj.__name__.lower()
            item_result[m_to_m_name] = []
            m_to_m = getattr(item, m_to_m_name).all()
            mapping = m_to_m_mapping[m_to_m_name]
            for instance in m_to_m:
                for key in mapping:
                    value = getattr(instance, mapping[key])
                    if value:
                        item_result[key].append(value)

    def extend_related_field(self, item, item_result, related_obj, related_mapping):
        if not related_obj:
            return
        for obj in related_obj:
            # 获取关联表
            related_name = obj.__name__.lower()
            # 获取queryset中的外键对象
            related_instance = getattr(item, related_name)
            # 对结果集进行字段扩展
            mapping = related_mapping[related_name]
            for key in mapping:
                child_obj = related_instance
                if child_obj:
                    # 多级 关联
                    if "." in mapping[key]:
                        attrs = mapping[key].split(".")
                        # 多级 关联的外键对象
                        child_obj = getattr(related_instance, attrs[0])
                        for attr in attrs[1:]:
                            child_obj = getattr(child_obj, attr)
                        item_result[key] = child_obj
                    else:
                        item_result[key] = getattr(related_instance, mapping[key], None)
                else:
                    item_result[key] = None

    def get_queryset(self, obj, data) -> (QuerySet, QuerySet):
        pass


class RelatedView(BaseView):
    relate = Related()
    injection_data = InjectionData()
    CHOOSE_GROUP = "choosegroup"
    RESERVED_FIELD = deepcopy(Select.RESERVED_FIELD) + [CHOOSE_GROUP]
    FORBID_VALUES = dict(**Select.FORBID_VALUES, **{CHOOSE_GROUP: ["password"]})

    @related_vaild_legal_decorator
    def get(
        self,
        request,
        obj,
        obj_serializer,
        *,
        extend_conditions: dict = None,
        need_results=False,
        need_queryset=False,
        keys=None,
        related_objs: list = None,
        m_to_m_objs: list = None,
        related_mapping: dict = None,
        m_to_m_mapping: dict = None,
        **kwargs,
    ):
        """
        扩展返回字段（关联表）
        1、获取queryset
        2、扩展字段

        """
        data = self.get_req_args(request, extend_conditions)
        results, queryset, current_page_queryset = self.find(obj, data, obj_serializer)

        try:
            if not related_objs:
                related_objs, related_mapping = self.injection_data.related_data("GET")
            if not m_to_m_objs:
                m_to_m_objs, m_to_m_mapping = self.injection_data.m_to_m_data("GET")
            if results.data and results.data.result:
                result = self.relate.extend_result_field(
                    obj_serializer, related_objs, m_to_m_objs, related_mapping, m_to_m_mapping, current_page_queryset
                )
                results.data.result = result
        except Exception as e:
            get_error_status_code(e, results)
        if need_results:
            return results
        return RESTResponse(results)

    def post(
        self,
        request,
        obj,
        obj_serializer,
        *,
        data=None,
        need_results=False,
        raise_exception=False,
        related_objs: list = None,
        m_to_m_objs: list = None,
        related_mapping=None,
        m_to_m_mapping=None,
        **kwargs,
    ):
        results = Results()
        if not data:
            data = request.data
        if data.get("data"):
            data = data.get("data")
        serializer = obj_serializer(data=data)
        if not related_objs:
            related_objs, related_mapping = self.injection_data.related_data("POST")
        if not m_to_m_objs:
            m_to_m_objs, m_to_m_mapping = self.injection_data.m_to_m_data("POST")
        self.relate.execute_related_extend(related_objs, related_mapping, data)
        self.relate.execute_m_to_m_extend(m_to_m_objs, m_to_m_mapping, data)
        try:
            valid = serializer.is_valid(raise_exception=True)
            serializer.save()
            results.describe = "add  successfully！！！"
            results.status = SUCCESS
            results.code = 200
        except Exception as e:
            if raise_exception:
                raise e
            results.code, results.describe = get_error_status_code(e)
        if need_results:
            return results
        return RESTResponse(results)

    def get_queryset(self, obj, data) -> (QuerySet, QuerySet):

        drop_group = data.get(self.CHOOSE_GROUP)
        queryset, extend_queryset = super().get_queryset(obj, data)
        if drop_group:
            group = tuple(drop_group.split(","))
            extend_queryset = queryset.values(*group).annotate(count=Count("id"))
        return queryset, extend_queryset

    def extend_page_type_results(self, data, page_type_results, extend_queryset, *args, **kwargs):
        if extend_queryset:
            page_type_results.group = list(extend_queryset)

    def put(
        self,
        request,
        obj,
        obj_serializer,
        *,
        data=None,
        pk="id",
        extend_conditions: dict = None,
        need_results=False,
        need_queryset=False,
        raise_exception=False,
        related_obj: list = None,
        m_to_m_obj: list = None,
        related_mapping=None,
        m_to_m_mapping=None,
        **kwargs,
    ):
        results = Results()
        if not data:
            data = request.data
        if data.get("data"):
            data = data.get("data")
        if not related_obj:
            related_obj, related_mapping = self.injection_data.related_data("PUT")
        if not m_to_m_obj:
            m_to_m_obj, m_to_m_mapping = self.injection_data.m_to_m_data("PUT")
        self.relate.execute_related_extend(related_obj, related_mapping, data)
        self.relate.execute_m_to_m_extend(m_to_m_obj, m_to_m_mapping, data)

        return super().put(
            request,
            obj,
            obj_serializer,
            data=data,
            pk=pk,
            extend_conditions=extend_conditions,
            need_results=need_results,
            need_queryset=need_queryset,
            raise_exception=raise_exception,
            **kwargs,
        )



#!/usr/bin/python3
import json
import socket
from threading import Thread
from time import sleep

from common.logging_config import logger

import websocket


class WebSocketClient:
    def conn_ws(self, host="127.0.0.1", port=5678):
        self.host = host
        self.port = port
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.connect((host, port))
        return self.sock

    def send_msg(self, msg, encoding="utf-8"):
        if isinstance(msg, dict) or isinstance(msg, list):
            msg = json.dumps(msg)
        logger.debug("send msg ", self.host, self.port, msg)
        if not isinstance(msg, bytes):
            msg = msg.encode(encoding)
        self.sock.send(msg)

    def recv_msg(self, encoding="utf-8"):
        while True:
            msg = self.sock.recv(1024)
            msg = msg.decode(encoding)
            logger.debug("recv msg", self.host, self.port, msg)


class WebSocketAppClient:
    def __init__(self, address, on_message=None, on_open=None, on_close=None, on_error=None, on_ping=None):
        websocket.enableTrace(False)
        self.ws = websocket.WebSocketApp(
            address,
            on_message=on_message if on_message else self.on_message,
            on_error=on_error if on_error else self.on_error,
            on_close=on_close if on_close else self.on_close,
            on_open=on_open if on_open else self.on_open,
            on_ping=on_ping if on_ping else self.on_ping,
        )

    def on_message(self, ws, message):
        logger.debug(message)

    def on_error(self, ws, error):
        logger.error("on_error错误：", error)

    def on_ping(self, ws):
        while True:
            ws.send(" ")
            sleep(30)

    def on_close(self, ws):
        logger.warning("websocket close!!!")

    def on_open(self, ws):
        logger.info("websocket open!!!")

    def run_forever(self):
        Thread(target=self._bg_run_forever).start()

    def _bg_run_forever(self):
        while True:
            self.ws.run_forever()
            sleep(10)


if __name__ == "__main__":
    client = WebSocketClient()
    client.conn_ws()
    Thread(target=client.recv_msg).start()
    while True:
        client.send_msg("示范法")
        sleep(2)



from copy import deepcopy

from common.logging_config import logger
from common.rest_extend.response import Results, PageTypeResults, SERVER_ERROR_CODE
from elasticsearch import Elasticsearch
from common.utility.args_parsing import BaseArgsParsing
from common.utility.base import to_utc_time


class ESClient(BaseArgsParsing):
    LIST_KEYWORD = ["must", "should"]
    RANGE = "range"

    def conn(self, host=None, port=None, username=None, password=None):
        if username and password:
            self.es = Elasticsearch(f"http://{host}/", port=port, http_auth=(username, password))
        else:
            self.es = Elasticsearch(f"http://{host}/", port=port)
        return self.es

    def transform(self, data):
        body = {}
        for key in data:
            if (not key) or (not data.get(key)):
                continue
            values = data.get(key)
            if key in self.RESERVED_FIELD:
                continue
            itme = key.split("_")
            if len(itme) > 1:
                real_key = "_".join(itme[:-1])
                keyword = itme[-1].lower()
                if keyword in self.SUFFIX_KEYWORD:
                    self.build_suffix_keyword_args(body, real_key, keyword, data.get(key))
                    continue

            page, size = self.get_page_and_size(data)

    def build_suffix_keyword_args(self, body, key, keyword, value):
        if keyword == self.GT or keyword == self.GTE or keyword == self.LT or keyword == self.LTE:
            if "query" in body.keys():
                pass
            else:
                body["query"] = {
                    "range": {
                        key: {
                            "gt": "2021-07-26T10:00:00Z",
                            "lt": "2021-07-26T10:02:00Z",
                        }
                    }
                }

    def build(self, body, keys, field, vaule):
        if body is None:
            body = {}
        if "." in keys:
            itme = keys.split(".")
            key = itme[0]
            if isinstance(body, dict):
                if key in body.keys():
                    body[key] = self.build(body[key], ".".join(itme[1:]), field, vaule)
                    return body
                else:
                    child_body = [] if key in self.LIST_KEYWORD else {}
                    body[key] = self.build(child_body, ".".join(itme[1:]), field, vaule)
                    return body
            else:
                if key == self.RANGE:
                    for range_item in body:
                        if key in range_item.keys():
                            if itme[1] in range_item[key].keys():
                                range_item[key][itme[1]] = self.build(
                                    range_item[key], ".".join(itme[1:]), field, vaule
                                )
                                break
                    else:
                        body.append({key: self.build({}, ".".join(itme[1:]), field, vaule)})
                else:
                    for item in body:
                        if key in item.keys():
                            item[key] = self.build(item[key], ".".join(itme[1:]), field, vaule)
                            break
                    else:
                        body.append({key: self.build({}, ".".join(itme[1:]), field, vaule)})
                return body

        else:
            key = keys
            if isinstance(body, dict):
                if key in body.keys():
                    body[key][field] = vaule
                    return body[key]
                else:
                    return {key: {field: vaule}}
            else:
                body.append({key: {field: vaule}})
                return body

    def build_range(self, body, real_field, kw, vaule):
        if body:
            pass
        return body

    def build2(self, body, keys, field, vaule):
        if body is None:
            body = {}
        if "." in keys:
            itme = keys.split(".")
            key = itme[0]
            if key in body.keys():
                body[key] = self.build(body[key], ".".join(itme[1:]), field, vaule)
                return body
            else:
                child_body = [] if key in self.LIST_KEYWORD else {}
                body[key] = self.build(child_body, ".".join(itme[1:]), field, vaule)
                return body

        else:
            key = keys
            if isinstance(body, dict):
                if key in body.keys():
                    body[key][field] = vaule
                    return body[key]
                else:
                    return {key: {field: vaule}}
            else:
                body.append({key: {field: vaule}})
                return body

    def sort_(self, key, value, body=None):
        if not body:
            body = {}
        sort_data = {key: {"order": value}}
        if "sort" in body.keys():
            body["sort"].append(sort_data)
        else:
            body["sort"] = [sort_data]
        return body

    def and_(self, key, value, body=None):

        # body = self.build(body, 'query.bool.must.match_phrase', key, value)
        body = self.build(body, "query.bool.must.match_phrase", key, value)
        return body

    def or_(self, key, value, body=None):

        # body = self.build(body, 'query.bool.should.match_phrase', key, value)
        body = self.build(body, "query.bool.should.match_phrase", key, value)
        return body

    def range_(self, key, kw, value, body=None):
        # body = self.build(body, 'query.bool.must.range.' + key, kw, value)
        body = self.build(body, "query.bool.must.range." + key, kw, value)
        return body

    def not_(self, key, value, body=None):
        body = self.build(body, "query.bool.must_not.match_phrase", key, value)
        return body

    def must_and_should(self, key, *, values: list = None, body=None):
        if not values:
            raise ValueError("'values' is none!!!")
        for value in values:
            if value:
                body = self.build(body, "query.bool.must.bool.should.match_phrase", key, value)
        return body

    def search(self, body, index, page=1, size=20, all_result=False):
        if all_result:
            results = self.es.search(body=body, index=index, size=10000)
        else:
            from_ = (page - 1) * size
            if page * size > 10000:
                raise ValueError(f"'page'*'size'不能大于10000，当前 {page * size}")
            results = self.es.search(body=body, index=index, size=size, from_=from_)
        return results

    def to_results(self, es_results, page, size, results: Results = None):
        if not results:
            results = Results()
        results.code = 200
        page_type_results = PageTypeResults()
        results.data = page_type_results
        try:
            for item in es_results["hits"]["hits"]:
                try:
                    _source = item["_source"]
                    _time = _source['time']
                    _time = to_utc_time(_time)
                    _source['time'] = _time
                    page_type_results.result.append(_source)
                except Exception as e:
                    logger.exception(e)
            page_type_results.page = page
            page_type_results.size = size
            page_type_results.total = es_results["hits"]["total"]["value"]
            results.data_length = len(page_type_results.result)
        except Exception as e:
            logger.exception(e)
            results.code = SERVER_ERROR_CODE
            results.describe = str(e)

        return results

    def index(self, index, body, id=None):
        if not body:
            return

        self.es.index(index, body=body, id=id)


if __name__ == "__main__":
    es_client = ESClient()
    body = es_client.and_("series", "P")
    es = es_client.conn(
        host="config.esmeas.host",
        port="config.esmeas.port",
        username="config.esmeas.username",
        password="config.esmeas.password",
    )
    es.index()



from common.rest_extend.decorator import tenant_decorator

from common.rest_extend.views.views import BaseView


class IntegrateView(BaseView):
    @tenant_decorator
    def get(
            self,
            request,
            obj,
            obj_serializer,
            *,
            data=None,
            extend_conditions: dict = None,
            need_results=False,
            need_queryset=False,
            keys=None,
            **kwargs,
    ):
        return super(IntegrateView, self).get(
            request,
            obj,
            obj_serializer,
            data=data,
            extend_conditions=extend_conditions,
            need_results=need_results,
            need_queryset=need_queryset,
            keys=keys,
            **kwargs,
        )

    @tenant_decorator
    def post(self, request, obj, obj_serializer, *, data=None, need_results=False, **kwargs):
        return super(IntegrateView, self).post(
            request, obj, obj_serializer, data=data, need_results=need_results, **kwargs
        )

    @tenant_decorator
    def put(
            self,
            request,
            obj,
            obj_serializer,
            *,
            data=None,
            pk="id",
            extend_conditions: dict = None,
            need_results=False,
            **kwargs,
    ):
        return super(IntegrateView, self).put(
            request,
            obj,
            obj_serializer,
            data=data,
            pk=pk,
            extend_conditions=extend_conditions,
            need_results=need_results,
            **kwargs,
        )

    @tenant_decorator
    def delete(
            self,
            request,
            obj,
            obj_serializer,
            *,
            data=None,
            pk="id",
            extend_conditions: dict = None,
            need_results=False,
            **kwargs,
    ):
        return super(IntegrateView, self).delete(
            request,
            obj,
            obj_serializer,
            data=data,
            pk=pk,
            extend_conditions=extend_conditions,
            need_results=need_results,
            **kwargs,
        )



from dataclasses import asdict
import json
from django.core.handlers.wsgi import WSGIRequest
from common.rest_extend.response import Results, get_error_status_code, RESTResponse, FORBID_CODE


def vaild_decorator(obj=None, obj_serializer=None, partial=False):
    """
    参数验证装饰器（这里当 middleware ，验证失败不往下执行，直接返回Response）
    :param obj: 实体类
    :param obj_serializer: 需要校验的序列化类
    :param partial: 是否部分校验
    :return:
    """

    def external_wrapper(function):
        def internal_wrapper(obj_request, request, *args, **kwargs):
            """

            :param obj_request: 被装饰的实体类
            :param request:
            :param args:
            :param kwargs:
            :return:
            """
            try:
                if request.method == "GET":
                    data = request.GET.dict()
                else:
                    data = request.data
                serializer = obj_serializer(data=data, partial=partial)
                valid = serializer.is_valid(raise_exception=True)
                kwargs["obj_ser"] = serializer
            except Exception as e:
                results = Results()
                results.describe = str(e)
                status_code, describe = get_error_status_code(e)
                return RESTResponse(asdict(results), status=status_code)
            return function(obj_request, request, *args, **kwargs)
            # response = function(obj_request, request, *args, **kwargs)
            # if json.loads(response.content).get('status') != SUCCESS:
            #     response.status = SERVER_ERROR_CODE
            # return response

        return internal_wrapper

    return external_wrapper


def view_decorator(function):
    """
    视图装饰器
    :param function:
    :return:
    """

    def wrapper(request, *args, **kwargs):
        return function(request, *args, **kwargs)

    return wrapper


def related_vaild_legal_decorator(func):
    def wrapper(obj, request, *args, **kwargs):
        # if request.method != "GET" and request.method != "POST":
        #     return JsonResponse({"detail": f'Method "{request.method}" not allowed.'}, status=405)
        group = request.GET.get("group")
        if group:
            results = Results()
            results.describe = "此接口不支持参数 'group'！！！"
            return RESTResponse(results)
        return func(obj, request, *args, **kwargs)

    return wrapper


def tenant_decorator(func):
    def wrapper(obj_view, request, *args, **kwargs):
        if isinstance(request, WSGIRequest):
            tenant = request.tenant
            try:
                if request.body:
                    data = json.loads(request.body)
                else:
                    data = {}
            except Exception as e:
                data = {}
        else:
            tenant = request._request.tenant
            data = request.data
        req_args = request.GET.dict()
        if request.method == 'GET':
            ignore_creator = req_args.pop('ignore_creator', "False").lower() == 'true'
            if ignore_creator:
                return func(obj_view, request, *args, **kwargs)

        if isinstance(data, dict):
            if 'data' in data.keys() and 'ops' in data.keys():
                data = data.get('data', {})
        if request.method == 'POST' or request.method == 'PUT':
            if isinstance(data, list):
                for item in data:
                    item['creator'] = tenant.name
            else:
                data['creator'] = tenant.name
            kwargs["data"] = data
        if request.method == 'GET':
            if "extend_conditions" in kwargs.keys():
                kwargs["extend_conditions"]['creator'] = tenant.name
            else:
                kwargs["extend_conditions"] = {"creator": tenant.name}
        return func(obj_view, request, *args, **kwargs)

    return wrapper

# def tenant_decorator(func):
#     def wrapper(obj_view, request, *args, **kwargs):
#         tenant = None
#         req_args = dict(request.GET.dict())
#         token = req_args.pop('token', None)
#         if request.method == 'GET':
#             data = req_args
#             ignore_creator = data.pop('ignore_creator', "False").lower() == 'true'
#             if ignore_creator:
#                 return func(obj_view, request, *args, **kwargs)
#         else:
#             data = request.data
#
#         if token:
#             tenant = Tenant.objects.filter(token=token).first()
#         else:
#             account = request.session.get('account')
#             if account:
#                 tenant = Tenant.objects.filter(account=account).first()
#         if tenant:
#             if isinstance(data, dict):
#                 if 'data' in data.keys() and 'ops' in data.keys():
#                     data = data.get('data', {})
#             if request.method == 'POST' or request.method == 'PUT':
#                 if isinstance(data, list):
#                     for item in data:
#                         item['creator'] = tenant.name
#                 else:
#                     data['creator'] = tenant.name
#                 kwargs["data"] = data
#             if request.method == 'GET':
#                 if "extend_conditions" in kwargs.keys():
#                     kwargs["extend_conditions"]['creator'] = tenant.name
#                 else:
#                     kwargs["extend_conditions"] = {"creator": tenant.name}
#             return func(obj_view, request, *args, **kwargs)
#         else:
#             results = Results()
#             results.code = FORBID_CODE
#             results.describe = "无登陆信息!!!"
#             return RESTResponse(results)
#
#     return wrapper



from rest_framework.views import APIView
from common.rest_extend.response import get_error_status_code, Results
from common.rest_extend.views.base import BaseORM



class BulkView(BaseORM, APIView):

    def post(self, request, obj, obj_serializer, *, data=None, need_results=False, **kwargs):
        results = Results()
        try:
            data = self.get_data(request, data)
            if not isinstance(data, list):
                raise ValueError("'data' type must be list!!!")
            objs = []
            for item in data:
                serializer = obj_serializer(data=item)
                serializer.is_valid(raise_exception=True)
                objs.append(obj(**item))
            obj.objects.bulk_create(objs)
        except Exception as e:
            get_error_status_code(e, results)

        return self.get_results(results, need_results=need_results)



import abc
import datetime

from common.rest_extend.views.base import Select, BaseORM
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import Q, Count
from django.db.models.query import QuerySet
from django.http import QueryDict
from rest_framework.exceptions import ValidationError
from rest_framework.views import APIView
from copy import deepcopy
from common.rest_extend.response import (
    Results,
    SUCCESS,
    FAILURE,
    REQUEST_ERROR_CODE,
    SERVER_ERROR_CODE,
    PageTypeResults,
    get_error_status_code,
    RESTResponse,
)
from common.utility.args_parsing import BaseArgsParsing


class ResponseJsonEncoder(DjangoJSONEncoder):
    def default(self, o):
        _o = super(ResponseJsonEncoder, self).default(o)
        if isinstance(o, datetime.datetime):

            return o.strftime("%Y-%m-%d %H:%M:%S")
        else:
            return _o


class BaseView(APIView, Select, BaseORM):
    """
    对实体模型增删改查的父类
    """

    def get(
            self,
            request,
            obj,
            obj_serializer,
            *,
            extend_conditions: dict = None,
            need_results=False,
            need_queryset=False,
            keys=None,
            **kwargs,
    ):
        """
        根据键值对查找数据
        :param request:
        :param obj:
        :param obj_serializer:
        :param data:
        :param need_results:
        :param keys:
        :return:
        """

        data = self.get_req_args(request, extend_conditions)
        results, queryset, current_page_queryset = self.find(obj, data, obj_serializer)
        if need_results:
            return results
        if need_queryset:
            return results, queryset
        return RESTResponse(results)

    def post(self, request, obj, obj_serializer, *, data=None, need_results=False, raise_exception=False, **kwargs):
        """
        添加数据

        """
        results = Results()
        if not data:
            data = request.data
        if data.get("data"):
            data = data.get("data")
        serializer = obj_serializer(data=data)
        try:
            valid = serializer.is_valid(raise_exception=True)
            serializer.save()
        except Exception as e:
            if raise_exception:
                raise e
            results.code, results.describe = get_error_status_code(e)
        else:
            results.describe = "add  successfully！！！"
            results.status = SUCCESS
            results.code = 200
        if need_results:
            return results
        return RESTResponse(results)

    def put(
            self,
            request,
            obj,
            obj_serializer,
            *,
            data=None,
            pk="id",
            extend_conditions: dict = None,
            need_results=False,
            need_queryset=False,
            raise_exception=False,
            **kwargs,
    ):
        instance = None
        results = Results()
        results.code = 200
        partial = True
        try:
            data, pk, value_pk, conditions = self.parsing_args_and_data(request, results, data, extend_conditions, pk)
            if results.code != 200:
                if need_queryset:
                    return results, None
                if need_results:
                    return results
                return RESTResponse(results)
            orm_q = self.get_orm_q(conditions)
            instance = obj.objects.filter(orm_q).first()
            if not instance:
                results.describe = pk + "=" + str(value_pk) + "  does not exist"
                results.code = REQUEST_ERROR_CODE
            else:
                serializer = obj_serializer(instance, data=data, partial=partial)
                try:
                    serializer.is_valid(raise_exception=True)
                    serializer.save()
                    results.describe = "update  successfully！！！"
                    results.status = SUCCESS
                except Exception as e:
                    results.code, results.describe = get_error_status_code(e)
                if getattr(instance, "_prefetched_objects_cache", None):
                    # If 'prefetch_related' has been applied to a queryset, we need to
                    # forcibly invalidate the prefetch cache on the instance.
                    instance._prefetched_objects_cache = {}

        except Exception as e:
            if raise_exception:
                raise e
            results.code, results.describe = get_error_status_code(e)

        if need_results:
            return results
        if need_queryset:
            return results, instance
        return RESTResponse(results)

    def delete(
            self,
            request,
            obj,
            obj_serializer,
            *,
            data=None,
            pk="id",
            extend_conditions: dict = None,
            need_results=False,
            need_queryset=False,
            raise_exception=False,
            **kwargs,
    ):
        instance = None
        results = Results()
        results.code = 200
        try:
            data, pk, value_pk, conditions = self.parsing_args_and_data(request, results, data, extend_conditions, pk)
            if results.code != 200:
                if need_results:
                    return results
                return RESTResponse(results)
            orm_q = self.get_orm_q(conditions)
            instance = obj.objects.filter(orm_q).first()
            if not instance:
                results.describe = pk + "=" + str(value_pk) + " does not exist"
                results.code = REQUEST_ERROR_CODE
            else:
                instance.delete()
                results.describe = "deleted successfully！！！"
                results.status = SUCCESS
        except Exception as e:
            if raise_exception:
                raise e
            results.code, results.describe = get_error_status_code(e)

        if need_results:
            return results
        if need_queryset:
            return results, instance
        return RESTResponse(results)

    def get_queryset(self, obj, data) -> (QuerySet, QuerySet):
        return super(BaseView, self).get_queryset(obj, data)

    def parsing_args_and_data(self, request, results, data, extend_conditions, pk):
        conditions = {}
        if not extend_conditions:
            extend_conditions = {}
        if not data:
            data = request.data
        if data.get("data"):
            data = data.get("data")
        if request.GET.get("pk"):
            pk = request.GET.get("pk")
        value_pk = data.pop(pk, None)
        if not value_pk:
            value_pk = request.GET.get(pk)
        if not value_pk:
            results.describe = "'pk' cannot be empty"
            results.code = REQUEST_ERROR_CODE
        else:
            try:
                if pk == "id":
                    try:
                        value_pk = int(value_pk)
                    except:
                        raise ValidationError("'pk' type error Should be <int>！！！")
                conditions = dict(**{pk: value_pk}, **extend_conditions)
            except Exception as e:
                results.code, results.describe = get_error_status_code(e)
        return data, pk, value_pk, conditions
